"""
    This file is part of Mokonnect.

    Mokonnect is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 3
    as published by the Free Software Foundation.

    Mokonnect is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Mokonnect.  If not, see <http://www.gnu.org/licenses/>.
"""
#
# mkdev_wifi.py
# mokonnect device wifi
#

import struct
import mkbase
# ugly
import time
import os

class ModePanel(mkbase.MKPanel):
	def __init__(self,win):
		mkbase.MKPanel.__init__(self,win)
		self.gui = {
			"type": "frame",
			"label": "Interface Configuration",
			"align": (-1.0,0.0),
			"content": {
				"type": "box",
				"content": [
					{"type":"check","label":"Default","config_link":"default"},
				]
			}
		}
		self.config = {"default":True}

class IPConfigPanel(mkbase.MKPanel):
	def __init__(self,win):
		mkbase.MKPanel.__init__(self,win)
		self.gui = {
			"name": "ipconfig",
			"type": "frame",
			"label": "IP Configuration",
			"align": (-1.0,0.0),
			"content": {
				"type": "table",
				"cols": 3,
				"content": [
					[
						{"type":"label","label":"IP Address"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"ip"}
					],
					[
						{"type":"label","label":"Netmask"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"netmask"}
					],
					[
						{"type":"label","label":"Gateway"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"gateway"}
					],
					[
						{"type":"label","label":"DNS Server"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"nameserver"}
					],
				]
			}
		}
		self.config = {"ip": "10.0.0.5","netmask":"255.255.255.0","gateway":"10.0.0.138","nameserver":"10.0.0.138"}

class ClientPanel(mkbase.MKPanel):
	def __init__(self,win,pager,bus,device,powerfunc):
		self.bus = bus
		self.pager = pager
		self.powerfunc = powerfunc
		self.device = device
		mkbase.MKPanel.__init__(self,win)
		self.gui = {
			"type": "frame",
			"label": "Network Configuration",
			"align": (-1.0,0.0),
			"content": {
				"type": "table",
				"cols": 3,
				"content": [
					[
						{"type":"button","label":"Scan","align":(-1,0),"clicked":self.ScanClicked},
					],
					[
						{"type":"label","label":"Mode"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type": "button","align":(-1,0),"config_link":"mode","save_object":"mode_button","clicked": self.ModeClicked},
					],
					[
						{"type":"label","label":"SSID"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"ssid","save_object":"net_name"}
					],
					[
						{"type":"label","label":"Channel"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"channel","save_object":"net_chan"}
					],
					[
						{"type":"label","label":"Authenticaion"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"button","config_link":"auth","save_object":"auth_button","clicked":self.AuthClicked}
					],
					[
						{"type":"label","label":"Password"},
						{"type":"box","align":(-1.0,0.0),"weight":(1,1)},
						{"type":"entry","config_link":"pass","save_object":"net_pass"}
					],
					[{"type":"check","label":"Use DHCP","config_link":"dhcp","panel_link":"ipconfig","panel_link_reverse":None}],
				]
			}
		}
		self.config = {"ssid": "ap_name","auth":"none","pass":"Password","channel":"11","dhcp":True,"mode":"managed"}
		# config shared hover panel
		self.hover = mkbase.MKHoverChoice(win,"Authentication",["none","wep","wpa"])
		self.hover.config = self.config
		# a hover sharing config
		self.mode_hover = mkbase.MKHoverChoice(win,"Wifi Mode",["managed","adhoc"])
		self.mode_hover.config = self.config
		# scan box
		self.btnlist = []
		self.scanning = False
		self.scanbox_config = {}
		self.scanbox_gui = {
			"type": "box",
			"content": [
				{"type":"scroller","bounce":(0,0),"align":(-1,-1),"weight":(1,1),"content": {
					"type": "frame",
					"label": "Scanning...",
					"save_object": "scanbox_frame",
					"align": (-1,0),
					"weight": (1,1),
					"content": {
						"type": "box",
						"align": (-1,0),
						"weight": (1,1),
						"save_object": "scanlistbox",
						"content": [] # filled dynamically by refresh networks
					}
				}},
				{"type":"button","label":"back","clicked":self.ScanboxBackClicked}
			]
		}
	
	def ScanboxBackClicked(self,*args,**kargs):
		print "ScanboxBackClicked"
		self.scanning = False
		self.pager.content_pop()
		
	def ModeClicked(self,*args,**kwargs):
		self.mode_hover.BuildHoverChoice(self.config["obj_mode_button"])

	def AuthClicked(self,*args,**kwargs):
		self.hover.BuildHoverChoice(self.config["obj_auth_button"])
	
	def _dummy_log(self,msg):
		print msg
		return
	
	def _GetDevice(self):
		# find wifi device
		self.device = None
		for dpath in self.bus["connman"]["/"]["Manager"]["Devices"]:
			device = self.bus["connman"][dpath]["Device"]
			if device["Type"] == "wifi":
				self.device = device

	def ScanClicked(self,*args,**kwargs):
		# check power state and see if we find the device
		if not self.powerfunc(self._dummy_log):
			return
		self._GetDevice()
		if None == self.device:
			print "Error: this should never happen!"
			return
		
		# build the gui and prepare the box
		scanbox_obj = mkbase.EFLBuildObject(self.win,self.scanbox_gui,self.scanbox_config)
		self.pager.content_push(scanbox_obj)

		self.bus["connman"].AddSignalFunction(self.DbusSignal)
		self.btnlist = []
		self.RefreshNetworks()
		self.scanning = True
		if self.device["Powered"] == False:
			self.device["Powered"] = True
		print self.device.GetProperties()
		try:
			self.device.ProposeScan()
		except Exception,e:
			print "Exception on proposescan: %s" % str(e)
	
	def RefreshNetworks(self):
		for b in self.btnlist:
			b[0].delete()
		self.btnlist = []
		for netpath in self.device["Networks"]:
			netprop = self.bus["connman"][netpath]["Network"].GetProperties()
			if not "Strength" in netprop:
				netprop["Strength"] = 0
			gui_netbtn = {
				"type": "button",
				"label": "%s (%s,%d%%)" % (netprop["Name"],netprop["WiFi.Security"],netprop["Strength"]),
				"align": (-1,0),
				"weight": (1,1),
				"clicked": self.NetworkSelected,
			}
			netbtn = mkbase.EFLBuildObject(self.win,gui_netbtn,{})
			self.btnlist.append([netbtn,netprop])
			self.scanbox_config["obj_scanlistbox"].pack_end(netbtn)
	
	def DbusSignal(self,*args,**kargs):
		if not self.scanning:
			return
		if kargs["signal"] == "PropertyChanged":
			if args[0] == "Scanning":
				if not args[1]:
					self.scanbox_config["obj_scanbox_frame"].label_set("Scan Complete")
				else:
					self.scanbox_config["obj_scanbox_frame"].label_set("Scanning...")
			if args[0] == "Networks":
				self.RefreshNetworks()
		
	def NetworkSelected(self,obj,event,*args,**kwargs):
		self.scanning = False
		# find button
		netprop = None
		for b in self.btnlist:
			if b[0] == obj:
				netprop = b[1]
		if not netprop:
			print "ERROR: Shouldnt happen! no net properties to match the button"
			return
		# we set the network data
		self.config["obj_net_name"].entry_set(netprop["Name"])
		self.config["obj_net_chan"].entry_set("%d" % netprop["WiFi.Channel"])
		self.config["obj_auth_button"].label_set(netprop["WiFi.Security"])
		self.config["obj_auth_button"].name_set(netprop["WiFi.Security"])
		self.config["obj_mode_button"].label_set(netprop["WiFi.Mode"])
		self.config["obj_mode_button"].name_set(netprop["WiFi.Mode"])
		
		# we pop the network selection screen
		self.pager.content_pop()
		
class WifiDevice(mkbase.MKDevice):
	def __init__(self,win,pager,bus):
		mkbase.MKDevice.__init__(self,win)
		self.name = "Wifi"
		self.bus = bus
		self.pager = pager
		self.powered = False
		self.device = None
		# panels
		self.panels = [ModePanel(win),ClientPanel(win,pager,bus,self.device,self.PowerOn),IPConfigPanel(win)]
		# vars
		self.looking_for_network = None
		self.active_timer = None
		self.timeout_connect = 60
		self.timeout_scan = 30
	
	def _GetDevice(self):
		# find wifi device
		self.device = None
		try:
			for dpath in self.bus["connman"]["/"]["Manager"]["Devices"]:
				device = self.bus["connman"][dpath]["Device"]
				if device["Type"] == "wifi":
					self.device = device
		except:
			pass

	def _GetBroadcast(self):
		ipb = socket.inet_aton(self.panels[2].config["ip"])
		maskb = socket.inet_aton(self.panels[2].config["netmask"])
		ipl = struct.unpack("!L",ipb)[0]
		maskl = struct.unpack("!L",maskb)[0]
		rmaskl = 0xFFFFFFFF & (~maskl)
		bcastl = ipl | rmaskl
		bcastb = struct.pack("!L",bcastl)
		return socket.inet_ntoa(bcastb)

	def _ConnectToNetwork(self,found_network):
		# got network in found_network, update its security
		log = self.log
		if "none" != self.panels[1].config["auth"]:
			found_network["WiFi.Passphrase"] = self.panels[1].config["pass"]
		# disconnect if already connected
		if found_network["Connected"]:
			log("Disconnecting from the current network...")
			found_network.Disconnect()
		# connect to it
		self.active_timer = mkbase.SetTimeout(self.timeout_connect,self._TimePassed)
		self.bus["connman"].AddSignalFunction(self.DbusSignal)
		log("Connecting to '%s'..." % self.panels[1].config["ssid"])
		if self.panels[1].config["dhcp"]:
			log("Configuring using DHCP...")
			found_network["IPv4.Method"] = "dhcp"
		else:
			log("Configuring with static information...")
			found_network["IPv4.Method"] = "static"
			print self.panels[2].config
			found_network["IPv4.Address"] = self.panels[2].config["ip"]
			found_network["IPv4.Netmask"] = self.panels[2].config["netmask"]
			found_network["IPv4.Gateway"] = self.panels[2].config["gateway"]
			found_network["IPv4.Nameserver"] = self.panels[2].config["nameserver"]
			found_network["IPv4.Broadcast"] = self._GetBroadcast()
		
		found_network.Connect()
	
	def _TimePassed(self,*args,**kargs):
		self.log("Timeout reached, connection failed...")
		self.log("__DONE__")
		self.active_timer = None
		return False
	
	def _ConnmanPowerOn(self,log):
		wifi_dev = self.bus["odeviced"]["/org/freesmartphone/Device/PowerControl/WiFi"]["PowerControl"]
		if not wifi_dev.GetPower():
			print "setting power to on using odeviced"
			wifi_dev.SetPower(True)
			time.sleep(5) # ugly hack
		self.device = None
		self._GetDevice()
		if self.device == None:
			# ugly hack, restarting connmand
			print "killing and re-running connmand"
			os.system("killall connmand ; connmand")
			time.sleep(5) # very ugly...
			self._GetDevice()
			if self.device == None:
				return False
		return True
		
	def PowerOff(self,log):
		wifi_dev = self.bus["odeviced"]["/org/freesmartphone/Device/PowerControl/WiFi"]["PowerControl"]
		if wifi_dev.GetPower():
			wifi_dev.SetPower(False)
			log("Wifi Device was powered off.")
			log("__DONE__")
			return True
		log("Wifi Device is already powered off.")
		log("__DONE__")
		return True

	def PowerOn(self,log):
		self._GetDevice()
		if None == self.device:
			log("Wifi device seems to be off, trying to power it on...")
			if not self._ConnmanPowerOn(log):
				log("Failed powering on the device or it was still not found in connman.")
				log("__DONE__")
				return False
		return True
		
	def Apply(self,log):
		self.log = log
		self.sent_done = False
		
		# check power state and see if we find the device
		if not self.PowerOn(log):
			return
		
		log("Wifi device: %s" % self.device["Interface"])
		# check if the current network exists in network list
		found_network = None
		netlist = self.device["Networks"]
		for netpath in netlist:
			netobj = self.bus["connman"][netpath]["Network"]
			if netobj["Name"] == self.panels[1].config["ssid"]:
				found_network = netobj
				break
		if not found_network:
			# create a new network
			print "Network '%s' not found, scanning for 30 seconds..." % self.panels[1].config["ssid"]
			log("Network '%s' is not currently visible." % self.panels[1].config["ssid"])
			log("Triggering scan for %d seconds..." % self.timeout_scan)
			self.active_timer = mkbase.SetTimeout(self.timeout_scan,self._TimePassed)
			self.looking_for_network = self.panels[1].config["ssid"]
			self.bus["connman"].AddSignalFunction(self.DbusSignal)
			try:
				self.device.ProposeScan()
			except Exception,e:
				print "Exception on proposescan: %s" % str(e)
			# relegating the rest of connection logic to the callback functions
			return
		# trigger connection	
		self._ConnectToNetwork(found_network)
	
	def DbusSignal(self,*args,**kargs):
		#print "DeviceSignal - %s - %s" % (str(args),str(kargs))
		if kargs["signal"] == "PropertyChanged":
			#print "Property changed! %s %s" % (str(args),str(kargs))
			if args[0] == "Connections":
				# find the connection belonging to this device...
				if self.sent_done:
					return
				dname = self.device.iface.object_path
				for conpath in args[1]:
					if dname in conpath:
						# found our connecion
						# cancel timeout
						if self.active_timer:
							self.active_timer.delete()
							self.active_timer = None
						# print con details and flag done
						con = self.bus["connman"][conpath]["Connection"]
						cprop = con.GetProperties()
						print cprop
						self.log("IP: %s" % cprop["IPv4.Address"])
						self.log("Netmask: %s" % cprop["IPv4.Netmask"])
						self.log("Gateway: %s" % cprop["IPv4.Gateway"])
						self.log("Nameserver: %s" % cprop["IPv4.Nameserver"])
						self.log("Done!")
						self.log("__DONE__")
						self.sent_done = True
						return
		# network lookup logic
		if self.looking_for_network != None:
			if kargs["signal"] == "PropertyChanged":
				if args[0] == "Networks":
					for netpath in args[1]:
						netobj = self.bus["connman"][netpath]["Network"]
						if netobj["Name"] == self.panels[1].config["ssid"]:
							# we found the network, cancel looking_for_network
							self.log("Network '%s' was found, proceeding..." % self.looking_for_network)
							self.looking_for_network = None
							# reset timer
							if self.active_timer:
								self.active_timer.delete()
								self.active_timer = None
							# connect
							self._ConnectToNetwork(netobj)
							break
		